/*
 * (c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 * Init.java
 *
 * Created on May 4, 2003, 9:14 AM
 */

package com.securitycentric.metacoretex;

import com.securitycentric.metacoretex.lib.*;
import com.securitycentric.metacoretex.interfaces.*;
import com.securitycentric.metacoretex.gui.*;
import com.securitycentric.metacoretex.xml.conf.*;

import javax.xml.bind.*;
import java.util.*;
import java.net.*;
import java.util.jar.*;
import java.io.*;


import java.awt.Toolkit;
import java.awt.Dimension;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 *
 * @author  visigoth
 */
public class Init {
    
    
    public static String version = new String("$Id: Init.java,v 1.41 2003/10/02 02:20:44 visigoth_6 Exp $");
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        
        splash = new SplashFrame();
        centerWindow(splash);
        splash.show();
        splash.setValue(10);
        
        genComponents();
        splash.setValue(30);
        
        userDir = System.getProperty("user.dir");
        
        loadGlobals();
        
        
        
        targVect.add(new Target("localhost"));
        splash.setValue(40);
        
        splash.setValue(60);
        
        try {
            ProbeToolz.loadProbesFromDir(new File(config.getProperty("probe.dir")), depVect);
        } catch (IOException e) {
            System.out.println(e.getMessage());
        }
        splash.setValue(70);
        
        processDepVect();
        
        splash.setValue(80);
        Iterator i = allProbeVect.iterator();
        while (i.hasNext()) {
            Probe tmpo = (Probe)i.next();
            System.out.println("Probe: " + tmpo.getName());
        }
        
        splash.setValue(100);
        
        //  Shameless...
        splash.setText("By visigoth <visigoth@securitycentric.com>");
        try {
            Thread.sleep(3000);
        } catch (InterruptedException ie) {
        }
        
        splash.setVisible(false);
        splash.dispose();
        
        mainWindow = new MainWindow();
        //FIXME get command line options working
        console=false;
        mainWindow.setSize(700,500);
        centerWindow(mainWindow);
        mainWindow.show();
        
        // For now, lets warn windows users
        if (System.getProperty("os.name").startsWith("Window"))
            Init.doMessage("Warning","Windows support is sketchy at best.\n"+
            "Be sure to review README.windows!");
        
    }
    
    /* Eventaully Bother with This
    private static void loadProperties() {
        try {
            File propFile = new File(userDir+"/conf/globals.prop");
            config.load(new FileInputStream(propFile));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
     */
    
    private static void saveProperties() {
        try {
            File propFile = new File(userDir+"/conf/mctx.properties");
            String saveMess = "Would you like to save your global \nconfig options?";
            if (! propFile.exists()) {
                if (askYesNo("Save", saveMess) == JOptionPane.YES_OPTION) {
                    config.store(new FileOutputStream(propFile), "Saved Globals-Delete this file to get back to defaults");
                    doMessage("Config Saved","Global configuration options are "+
                    "being saved to:\n"+userDir+"/conf/mctx.properties");
                }
            }
            
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    
    private static void genComponents() {
        
        allProbeVect = new Vector();
        targVect = new Vector();
        attackVect = new Vector();
        reportVect = new Vector();
        depVect = new Vector();
        config = new Properties();
        
        try {
            confctx = JAXBContext.newInstance("com.securitycentric.metacoretex.xml.conf");
            
        } catch (JAXBException je) {
            splash.setText("JAXB Exception!!");
        }
    }
    
    public static void fire() {
        clearProbeReports();
        genAttackers();
        try {
            probeTimeout = Integer.parseInt(config.getProperty("probe.timeout"));
        } catch (NumberFormatException ne) {
            doError("User Specified bad number in global config",ne);
            return;
        }
        safeChecks = Boolean.getBoolean(config.getProperty("safe.checks.only"));
        Iterator i = attackVect.iterator();
        
        while (i.hasNext())
            ((Attacker)i.next()).start();
    }
    
    public static void stopAttacker(int x) {
        ((Thread)attackVect.get(x)).interrupt();
    }
    
    public static void clearProbeReports() {
        Iterator it = reportVect.iterator();
        while (it.hasNext()) {
            Target tar = (Target)it.next();
            tar.clearProbeReports();
            tar.clearProbeExceptions();
            tar.clearKb();
        }
        reportVect.clear();
        
    }
    
    public static void scanDone() {
        TLock.unlockAll();
        mainWindow.repFrame.moveToFront();
        mainWindow.toFront(mainWindow.repFrame);
    }
    
    private static void genAttackers() {
        int threads;
        int threadMax;
        try {
            threadMax = Integer.parseInt(config.getProperty("threads.max"));
        } catch (NumberFormatException ne) {
            JOptionPane.showMessageDialog(null, "User Specified bad threads.max : "+ config.getProperty("threads.max"), "Error", JOptionPane.ERROR_MESSAGE);
            threadMax = 0;
        }
        
        int targCount = targVect.size();
        
        attackVect.clear();
        threadGroup = new ThreadGroup("Attacker Group");
        
        if (targCount < threadMax)
            threads = targCount;
        else
            threads = threadMax;
        
        for (int i = 0; i<threads; i++) {
            attackVect.add(new Attacker(threadGroup));
        }
        
    }
    
    /* process the depVect Vector of probes to be indexed/loaded.
     * FIXME this should be capable of moving past a single probe
     * loader failure (which will currentl cause all others behind
     * it to fail...
     */
    public static boolean processDepVect() {
        
        int vectSize, newSize, index;
        do {
            vectSize = depVect.size();
            Iterator i = depVect.iterator();
            while (i.hasNext()) {
                Probe tmpPrb = (Probe)i.next();
                
                /* Always remove on reload of same ID */
                int id = tmpPrb.getProbeId();
                if (ProbeToolz.isLoaded(allProbeVect, id))
                    ProbeToolz.removeProbe(id);
                
                index = ProbeToolz.getIndex(allProbeVect, tmpPrb);
                if (index != -1) {
                    if (splash != null)
                        splash.setText(tmpPrb.getName());
                    
                    loadProbe(tmpPrb, index);
                    depVect.remove(tmpPrb);
                    break;
                }
            }
            
            newSize = depVect.size();
            
        /*
         * This will cause a loop for processing depVect which continues
         * until the vector ceases to get smaller in a pass
         */
        } while (newSize < vectSize);
        
        System.out.println("Had " + newSize + " remaining probes");
        // Remove stragglers
        depVect.clear();
        if (newSize == 0)
            return(true);
        else
            return(false);
        
    }
    
    /** loadProbe will load a probe at the specified index and
     * populate the interface probe vectors
     */
    public static void loadProbe(Probe pr, int index) {
        
        allProbeVect.add(index,pr);
        
        /* If we don't already have a vector for this family, make one */
        if (! probeVectHash.containsKey(pr.getFamily())) {
            probeVectHash.put(pr.getFamily(), new Vector());
        }
        
        ((Vector)probeVectHash.get(pr.getFamily())).add(pr);
        
    }
    
    public static void fireStatusChanged() {
        mainWindow.updateScanStatus();
    }
    
    public static void centerWindow(java.awt.Component frame) {
        Dimension frameSize = frame.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    }
    
    public static void ShutDown() {
        saveProperties();
        System.exit(0);
    }
    
    public static void LoadConfig(File xmlConf) {
        try {
            
            Unmarshaller um = confctx.createUnmarshaller();
            MCTXConfig cfg = (MCTXConfig)um.unmarshal(xmlConf);
            
            // Load up the Global Options
            Iterator gloIter = cfg.getGlobalOption().iterator();
            while (gloIter.hasNext()) {
                MCTXOption opt = (MCTXOption)gloIter.next();
                config.setProperty(opt.getName(), opt.getValue());
            }
            
            // Load up the Targets
            targVect.clear();
            Iterator tarIter = cfg.getTarget().iterator();
            while (tarIter.hasNext()) {
                MCTXTarget mctxtar = (MCTXTarget)tarIter.next();
                Target tmpTarg = new Target(mctxtar.getHost());
                targVect.add(tmpTarg);
            }
            
            // Iterate through probe configs
            Iterator prcfgIter = cfg.getProbeConfig().iterator();
            while( prcfgIter.hasNext()) {
                MCTXProbeConfig pcfg = (MCTXProbeConfig)prcfgIter.next();
                Probe prb = ProbeToolz.getProbeById(pcfg.getProbeId());
                
                // Config option for non-existant probe!
                if (prb == null) {
                    System.out.println("Config option for non-existant probe!");
                    continue;
                }
                // Iterate through options for this probe
                Iterator proptIter = pcfg.getProbeOption().iterator();
                while (proptIter.hasNext()) {
                    MCTXOption propt = (MCTXOption)proptIter.next();
                    prb.addOption(propt.getName(), propt.getValue());
                }
                
                prb.setEnabled(pcfg.isProbeEnabled());
            }
            
        } catch (JAXBException je) {
            doError("Unmarshaller Error", je);
        }
    }
    
    public static void SaveConfig(File xmlConf) {
        try {
            xmlConf.createNewFile();
            Marshaller m = confctx.createMarshaller();
            m.setProperty("jaxb.formatted.output",new Boolean(true));
            ObjectFactory fac = new ObjectFactory();
            
            MCTXConfig cfg = fac.createConfig();
            
            // Schlorp the global options
            Iterator gloIt = config.keySet().iterator();
            while (gloIt.hasNext()) {
                String key = (String)gloIt.next();
                MCTXOption opt = fac.createMCTXOption();
                opt.setName(key);
                opt.setValue(config.getProperty(key));
                cfg.getGlobalOption().add(opt);
            }
            
            // Iterate through probes (and options)
            Iterator probIt = allProbeVect.iterator();
            while (probIt.hasNext()) {
                Probe probe = (Probe)probIt.next();
                Vector propts = probe.getOptionKeys();
                
                MCTXProbeConfig prbcfg = fac.createMCTXProbeConfig();
                prbcfg.setProbeId(probe.getProbeId());
                prbcfg.setProbeVersion(probe.getVersion());
                Iterator proptIt = propts.iterator();
                
                while (proptIt.hasNext()) {
                    String key = (String)proptIt.next();
                    MCTXOption opt = fac.createMCTXOption();
                    opt.setName(key);
                    opt.setValue(probe.getOption(key));
                    prbcfg.getProbeOption().add(opt);
                }
                
                prbcfg.setProbeEnabled(probe.isEnabled());
                cfg.getProbeConfig().add(prbcfg);
            }
            
            // Iterate through the targets
            Iterator targIt = targVect.iterator();
            while (targIt.hasNext()) {
                MCTXTarget mctxtarg = fac.createMCTXTarget();
                Target targ = (Target)targIt.next();
                
                mctxtarg.setHost(targ.getHost());
                
                cfg.getTarget().add(mctxtarg);
                
            }
            OutputStream os = new FileOutputStream(xmlConf);
            m.marshal( cfg, os );
            
        } catch (JAXBException je) {
            je.printStackTrace();
        } catch (FileNotFoundException fe) {
            fe.printStackTrace();
        } catch (IOException io) {
            io.printStackTrace();
        }
    }
    
    public static void loadTargetFile(File file) {
        try {
            BufferedReader br = new BufferedReader(new FileReader(file));
            String line;
            
            while ((line = br.readLine()) != null) {
                if ( line.startsWith("#"))
                    continue;
                
                targVect.add(new Target(line.trim()));
            }
        } catch (IOException ie) {
            JOptionPane.showMessageDialog(null, ie.getMessage(), "I/O Exception", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    /* This method allows us a uniform way to handle errors and
     * makes a little more room for eventual headless runntime
     */
    public static void doError(String mess, Exception e) {
        if (console) {
            System.out.println(mess+"\n" + e.getMessage());
            e.printStackTrace();
        } else {
            JOptionPane.showMessageDialog(mainWindow, mess+" : "+e.getMessage(),"Error!", JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        }
    }
    
    public static void doMessage(String title, String mess) {
        if (console) {
            System.out.println(mess);
        } else {
            JOptionPane.showMessageDialog(mainWindow,mess, title, JOptionPane.INFORMATION_MESSAGE);
        }
    }
    
    public static int askYesNo(String title, String mess) {
        if (console) {
            // FIXME console ask and read
            return(JOptionPane.NO_OPTION);
        } else {
            return(JOptionPane.showConfirmDialog(null,mess,title,JOptionPane.YES_NO_OPTION));
        }
    }
    
    private static void loadGlobals() {
        System.setProperty("javax.net.ssl.trustStore", userDir+"/.truststore");
        // This will just contain the public CA cert, no biggie.
        System.setProperty("javax.net.ssl.trustStorePassword","changeit");
        // We'll probably need this eventually...
        //System.setProperty("javax.net.ssl.keyStore", userDir+"/.keystore");
        
        config.setProperty("threads.max","5");
        config.setProperty("safe.checks.only","true");
        config.setProperty("probe.timeout","60");
        config.setProperty("update.url","https://xml.metacoretex.com:8443/MetaWeb/Update");
        config.setProperty("probe.compiler.classpath",userDir+"/dist/MetaCoreTex.jar");
        config.setProperty("probe.dir",userDir+"/probes/");
        config.setProperty("conf.dir",userDir+"/conf/");
        config.setProperty("jar.dir",userDir+"/dist/");
        config.setProperty("template.dir",userDir+"/docs/templates/");
        
        try {
            File propFile = new File(userDir+"/conf/mctx.properties");
            if (propFile.exists()) {
                config.load(new FileInputStream(propFile));
            }
        } catch (IOException e) {
            doError("Error Loading Global Defaults",e);
        }
    }
    
    
    public static Probe selectedProbe;
    public static int probeTimeout;
    public static boolean safeChecks;
    public static boolean console = true;
    public static String userDir;
    
    public static Vector allProbeVect;
    public static Vector reportVect;
    public static Vector depVect;
    
    public static Hashtable probeVectHash = new Hashtable();
    
    // A ThreadGroup for our attackers
    public static ThreadGroup threadGroup;
    
    // A vector of targets
    public static Vector targVect;
    
    // A static global knowledge base
    public static Properties config;
    
    // Hmmm... What could this be?
    public static MainWindow mainWindow;
    public static SplashFrame splash;
    public static Dimension screenSize;
    
    public static Vector attackVect;
    public static JAXBContext confctx;
}
