/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * UpdateTableModel.java
 *
 */

package com.securitycentric.metacoretex.adapters;

import com.securitycentric.metacoretex.*;
import com.securitycentric.metacoretex.interfaces.*;
import com.securitycentric.metacoretex.lib.*;
import com.securitycentric.metacoretex.xml.update.*;

import java.util.*;
import java.util.jar.*;
import java.io.*;
import javax.swing.table.AbstractTableModel;

/**
 *
 * @author  visigoth
 * @version
 */
public class UpdateTableModel extends AbstractTableModel {
    
    public UpdateTableModel(MCTXUpdate upDate, int which) {
        
        this.which = which;
        // If which is 1, probes, if 2 confs
        switch (which) {
            case 1:
                curList = upDate.getProbe();
                break;
            case 2:
                curList = upDate.getConfFile();
                break;
            case 3:
                curList = upDate.getJar();
        }
        this.upDate = upDate;
        colNames = new String [] {"Name","Current","Available","Update?"};
        
    }
    
    public int getRowCount() {
        
        return (curList.size());
        
    }
    
    public int getColumnCount() {
        return(4);
    }
    
    public String getColumnName(int column) {
        return(colNames[column]);
    }
    
    public Class getColumnClass(int column) {
        switch(column) {
            case 3:
                return(Boolean.class);
        }
        return(String.class);
    }
    
    public Object getValueAt(int row, int column) {
        
        switch (which) {
            case 1:
                MCTXProbe prb = (MCTXProbe)curList.get(row);
                switch (column) {
                    case 0:
                        return(prb.getName());
                    case 1:
                        Probe realPrb = ProbeToolz.getProbeById(prb.getId());
                        if (realPrb != null)
                            return(realPrb.getVersion());
                        return("none");
                    case 2:
                        return(prb.getVersion());
                    case 3:
                        return(new Boolean(prb.isUpdate()));
                }
                break;
                
            case 2:
                MCTXConfFile cnf = (MCTXConfFile)curList.get(row);
                switch (column) {
                    case 0:
                        return(cnf.getName());
                    case 1:
                        File ourFile = new File(Init.config.get("conf.dir")+cnf.getName());
                        Version ourVer = ProbeToolz.confFileVersion(ourFile);
                        if (ourVer != null)
                            return(ourVer.toString());
                        return("none");
                    case 2:
                        return(cnf.getVersion());
                    case 3:
                        return(new Boolean(cnf.isUpdate()));
                }
                break;
                
            case 3:
                MCTXJar jar = (MCTXJar)curList.get(row);
                switch(column) {
                    case 0:
                        return(jar.getName());
                    case 1:
                        try {
                        JarFile ourJar = new JarFile(Init.config.get("jar.dir")+jar.getName());
                        Attributes attr = ourJar.getManifest().getMainAttributes();
                        Version ourVer = new Version(attr.getValue("MCTX-Jar-Version"));
                        
                        if (ourVer != null)
                            return(ourVer.toString());
                        
                        } catch (Exception e) {
                            return("none");
                        }
                        return("none");
                    case 2:
                        return(jar.getVersion());
                    case 3:
                        return(new Boolean(jar.isUpdate()));
                }
                break;
        }
        return(new String("WTF?"));
        
    }
    
    public boolean isCellEditable(int row, int col) {
        if (col == 3)
            return(true);
        
        return(false);
    }
    
    public void setValueAt(Object obj, int row, int col) {
        boolean bool;
        switch (which) {
            case 1:
                MCTXProbe prb = (MCTXProbe)curList.get(row);
                bool = ((Boolean)obj).booleanValue();
                prb.setUpdate(bool);
                break;
                
            case 2:
                MCTXConfFile cnf = (MCTXConfFile)curList.get(row);
                bool = ((Boolean)obj).booleanValue();
                cnf.setUpdate(bool);
                break;
        }
    }
    
    
    public List curList;
    public String[] colNames;
    public MCTXUpdate upDate;
    int which = 1; //Which updates to show. 1=probes, 2=configs
}
