/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * AbstractAuthProbe.java
 *
 * Created on May 5, 2003, 7:26 AM
 */

package com.securitycentric.metacoretex.lib;

import com.securitycentric.metacoretex.*;
import com.securitycentric.metacoretex.lib.*;
import com.securitycentric.metacoretex.interfaces.*;

import java.io.*;
import java.util.*;

/** This is the abstract probe from which new AuthProbes may be extended.  Most often
 * this type of probe is used if you would like to create a "brute forcer" which
 * can't use the same normal timeout as other probes.
 * @author visigoth
 */
public abstract class AbstractAuthProbe extends AbstractProbe implements AuthProbe {
    
    /** used by the thread timers to decide if this probe gets treated like a brute forcer
     * or a normal probe.
     * @return <CODE>true</CODE> if its a brute-force probe <CODE>false</CODE> if not
     */    
    public boolean isBrute() {
        return(brute);
    }
    
    /** Set the brute boolean value for this probe.  Used by implementing probes to
     * declare themselves a bruter-forcer.
     * @param b <CODE>true</CODE> if we brute, <CODE>false</CODE> if we don't
     */    
    public void setBrute(boolean b) {
        brute = b;
    }
    
    /** A method for implemeting AuthProbes which only require the test() method
     *  over-ride this method to implement your own
     */
    public void probe() throws ProbeException {
        int broken = 0;
        /* Call the prepare() method to give implementations a chance to
         * get ready before we begin to call test().  If it returns false,
         * this probe doesn't need to run.
         */
        if (! shouldRun())
            return;
        
        try {
            String pass = "none";
            BufferedReader br = new BufferedReader(new FileReader(getOption("dictionary")));
            while ((pass = br.readLine()) != null) {
                if (pass.startsWith("#"))
                    continue;
                
                broken += test(pass);
            }
        } catch (FileNotFoundException e) {
            throw(new ProbeException("File not found: "+getOption("dictionary")));
        } catch (IOException e) {
            throw(new ProbeException(e.getMessage()));
        }
         addReport("=============================");
         addReport("A total of "+broken+" accounts were broken");
         if (broken > 0) {
             setReportable(true);
         }
    }
    
    /** This method is a place-holder for those wishing to implement
     *  the probe() method directly and not use test() at all
     */
    public int test(String pass) throws ProbeException {
        return(0);
    }
    
    /** This method is a place holder for those wishing to implement
     *  the probe() method directly and won't need prepare()
     */
    public boolean shouldRun() throws ProbeException {
        //Let things run if they don't need to prepare
        return(true);
    }
    
    /** distinguishes weather or not this probe is a brute force probe.
     */    
    public boolean brute = false;
}
