/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * AbstractCommandProbe.java
 *
 * Created on May 4, 2003, 8:03 AM
 */

package com.securitycentric.metacoretex.lib;

import com.securitycentric.metacoretex.interfaces.*;
import com.securitycentric.metacoretex.lib.*;

import java.io.*;

/** Command Probes are used to run an operating system command or tool agains the
 * target system and store the output in the KB/Report.  Probes of this type should
 * add their information under the KB key "command/asdfasdf" where asdfasdf is the
 * name of the command.  STDOUT output from the command will be automatically added
 * to the report.  This is one of the few probe types which don't require the
 * developer to implement their own probe() method.
 * @author visigoth
 */
public abstract class AbstractCommandProbe extends AbstractProbe implements CommandProbe {
    

    /** Get a String of the command which this probe will execute on probe().
     * @return String of the command to execute.
     */    
    public String getCommand() {
        return(cmd);
    }

    /** Return the arguments string for this command probe.
     */    
    public String getArgs() {
        return(args);
    }
    
    /** Set the cmd string which is used to exec the probe
     * @param cmd name of the command to set for execution
     */    
    public void setCommand(String cmd) {
        this.cmd = cmd;
    }

    /** Set the arguments to the command.  Developers may insert the string _TARGET_
     * anywhere they would like to have the results from <CODE>getHost()</CODE>
     * inserted into the argument string.
     * @param args String like "-v -oX - _TARGET_" <-- extra credit, what app is this for?
     */    
    public void setArgs(String args) {
        this.args = args;
    }

    /** Set the KB key which this probe will use to store the STDOUT results of the
     * command on execution.
     * @param key String like "command/nmap"
     */    
    public void setKbKey(String key) {
        this.key = key;
    }

    /** Get the KB key (String) which is set by this probe.
     * @return String of the KB key used by this probe
     */    
    public String getKbKey() {
        return(key);
    }

    /** This Abstract probe implements is't own probe() method, because command probes
     * are just regular commands who'se output ends up in the report.
     */    
    public void probe() throws ProbeException {

        String fullStr = getOption("command")+" "+getOption("args");
        String line;
        String response = new String();
        
        fullStr = fullStr.replaceAll("_TARGET_",getHost());
        
        try {

            Process proc = Runtime.getRuntime().exec(fullStr);
            DataInputStream dis = new DataInputStream(proc.getInputStream());

            while ((line = dis.readLine()) != null) {
                response = response+line+"\n";
            }

            // Made it this far, lets put it all in
            setReportable(true);
            addReport(response);

            if (! key.matches("none"))
                kbPut(key,response);

        } catch (IOException e) {
            throw(new ProbeException(e.getMessage()));
        }
    }
    
    /** the command to run
     */    
    public String cmd = "java";
    /** command line arguments for cmd
     */    
    public String args = "-version";
    /** the KB key to use in setting this information in the KB
     */    
    public String key = "none";
}
