/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * AbstractDatabaseProbe.java
 *
 * Created on May 4, 2003, 8:03 AM
 */

package com.securitycentric.metacoretex.lib;

import com.securitycentric.metacoretex.interfaces.*;
import com.securitycentric.metacoretex.lib.*;

import java.sql.*;

/** AbstractDatabasrProbe extends the AbstractProbe to implement a couple database
 * friendly functions.
 * @author visigoth
 */
public abstract class AbstractDatabaseProbe extends AbstractProbe implements DatabaseProbe {

    /** Add a result set to the current report string.  This will set up a tab delimited
     * print out of the <CODE>ResultSet</CODE> by listing column names at the top.
     * @param res java.sql.ResultSet from a recent query.
     */    
    public void addResultSet(ResultSet res) throws SQLException {
        ResultSetMetaData meta = res.getMetaData();
        int cols = meta.getColumnCount();
        String line = new String();

        addReport("\n===========================");
        
        for (int i = 1; i<=cols; i++) {
            line = line+meta.getColumnName(i)+"\t";
        }
        addReport(line);
        addReport(" ");

        while(res.next()) {
            line = new String();
            for (int i=1; i<=cols; i++) {
                line = line+res.getString(i)+"\t";
            }
            addReport(line);
        }
        
        /* At least try to be a good neighbor */
        try {
            if (res.getType() != res.TYPE_FORWARD_ONLY)
                res.beforeFirst();
        } catch (SQLException e) {
        }
    }

}
