/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * ProbeToolz.java
 *
 * Created on May 4, 2003, 11:45 AM
 */

package com.securitycentric.metacoretex.lib;

import com.securitycentric.metacoretex.interfaces.*;
import com.securitycentric.metacoretex.*;

import javax.swing.JOptionPane;
import java.util.*;
import java.net.*;
import java.util.jar.*;
import java.io.*;


/**
 *
 * @author  visigoth
 */
public class ProbeToolz {
    
    public static void setProbesEnabled(Vector vect, boolean enabled) {
        Iterator i = vect.iterator();
        while (i.hasNext()) {
            Probe prb = (Probe)i.next();
            prb.setEnabled(enabled);
        }
    }
    
    
    /* I suppose we can put the tools for playing with config
     * files in here, as the configs belong to the probes
     */
    public static Version confFileVersion(File confFile) {
        try {
            BufferedReader br = new BufferedReader(new FileReader(confFile));
            String line = br.readLine().trim();
            Version ver = new Version(line);
            return(ver);
        } catch (IOException e) {
            /*
             * it should be ok to let this go quietly, the
             * only files being handed to this were just
             * stat'ed by a dir File
             */
            //e.printStackTrace();
        } catch (VersionException e) {
            /* This probably means file has been
             * modified by user.
             */
        }
        return(null);
    }
    
    /*
     * load any classes which implement the Probe interface from
     * the class files in the given directory
     */
    public static void loadProbesFromDir(File prDir, Vector vect) throws IOException{
        
        if (! prDir.isDirectory())
            throw(new IOException(prDir.getName()+" is not a Directory!"));
        
        String[] names = prDir.list(new FilenameFilter() {
            // Inline FilenameFilter
            public boolean accept(File dir, String name) {
                if (name.endsWith(".class"))
                    return(true);
                
                return(false);
            }
        });
        
        for (int i = 0; i < names.length; i++) {
            Probe prb = loadProbeFromFile(new File(prDir.getPath()+"/"+names[i]));
            if (prb != null)
                vect.add(prb);
        }
    }
    
    public static Probe loadProbeFromFile(File prFile) {
        
        if (! prFile.exists())
            return(null);
        
        try {
            
            String path = prFile.getParent();
            
            String className = prFile.getName().substring(0,prFile.getName().indexOf('.'));
            
            ProbeClassLoader pldr = new ProbeClassLoader(path+"/");
            Class probClass = pldr.loadClass(className);
            Probe newProbe = (Probe)probClass.newInstance();
            return(newProbe);
            
        } catch (Exception ex) {
            System.out.println("loadProbeFromFile threw: "+ ex.getClass().toString()+" : "+ex.getMessage());
        }
        return(null);
    }
    
    public static String getNameById(int id) {
        String name = "Probe Id Not Set!";
        
        Probe probeTmp = getProbeById(id);
        
        if (probeTmp == null)
            return(name);
        
        return(probeTmp.getName());
    }
    
    public static Probe getProbeById(int id) {
        Iterator piter = Init.allProbeVect.iterator();
        while (piter.hasNext()) {
            
            Probe probeTmp = (Probe)piter.next();
            
            if (probeTmp.getProbeId() == id)
                return(probeTmp);
        }
        return(null);
    }
    
    public static boolean isLoaded(Vector probeVect, int probeId) {
        // Test if we already have this probe somewhere, if so
        // load the new instance over the top of this
        Iterator i = probeVect.iterator();
        while (i.hasNext()) {
            Probe pTmp = (Probe)i.next();
            if (pTmp.getProbeId() == probeId)
                return(true);
        }
        return(false);
    }
    
    /** Remove all vector references to the given probe for
     * the same probeId
     */
    //FIXME shouldn't this include the allProbeVect?
    public static void removeProbe(int probeId) {
        Hashtable tab = Init.probeVectHash;
        
        Iterator famIter = tab.keySet().iterator();
        while (famIter.hasNext()) {
            String fam = (String)famIter.next();
            Vector famVect = (Vector)tab.get(fam);
            Iterator vectIter = famVect.iterator();
            while(vectIter.hasNext()) {
                Probe tmpPrb = (Probe)vectIter.next();
                if (tmpPrb.getProbeId() == probeId) {
                    vectIter.remove();
                    break;
                }
            }
        }
    }
    
    /* This mess gets the index at which a probe should be inserted
     * in order to meet id't depends/provides
     */
    public static int getIndex(Vector probeVect, Probe probe) {
        
        // if it provides nothing, put it at the back
        if (probe.provides().size() == 0)
            return(probeVect.size());
        
        // if it depends on nothing put it at the front
        if (probe.depends().size() == 0)
            return(0);
        
        int depIndex = getDepIndex(probeVect, probe);
        int provIndex = getProvIndex(probeVect, probe);
        // Provides something which isn't here yet
        if (provIndex == 0)
            return(depIndex);
        
        /* This will return either a valid depIndex entry,
         * or -1 if it needs to be processed again later
         */
        if (depIndex <= provIndex) {
            return(depIndex);
        }
        
        //If we get to here, theres a dep problem
        System.out.println("Dep Failure: depIndex = " + depIndex + " provIndex = " + provIndex + "depIndex must be <= provIndex");
        return(-1);
    }
    
    /** This will determine the EARLIEST in the vector this probe
     *  may go in order to meet it's dependencies, return -1 if
     *  providor is not yet loaded.
     *
     */
    public static int getDepIndex(Vector probeVect, Probe probe) {
        int index = -1;
        Iterator probeIter = probeVect.iterator();
        
        while (probeIter.hasNext()) {
            Probe ptmp = (Probe)probeIter.next();
            
            if (testDepends(probe,ptmp)) {
                // Increment one for the slot we are going to bump
                index = probeVect.indexOf(ptmp)+1;
            }
        }
        
        return(index);
    }
    
    /** Determine the last index which will let this probe
     *  provide for it's dependants (first if it's dependants aren't here)
     */
    public static int getProvIndex(Vector probeVect, Probe probe) {
        int index = 0;
        Iterator probeIter = probeVect.iterator();
        
        while (probeIter.hasNext()) {
            Probe ptmp = (Probe)probeIter.next();
            
            if (testDepends(ptmp, probe))
                index = probeVect.indexOf(ptmp);
        }
        
        return(index);
    }
    
    //  Does depOne depend on provOne?
    public static boolean testDepends(Probe depOne, Probe provOne) {
        Hashtable provHash = provOne.provides();
        
        Iterator depIter = depOne.depends().iterator();
        while (depIter.hasNext()) {
            String key = (String)depIter.next();
            if (provHash.containsKey(key))
                return(true);
        }
        
        return(false);
    }
    
    /* Return the next available probe id
     */
    public static int nextProbeId() {
        Iterator i = Init.allProbeVect.iterator();
        int id = 0;
        while (i.hasNext()) {
            Probe x = (Probe)i.next();
            if (x.getProbeId() > id)
                id = x.getProbeId();
        }
        id++;
        return(id);
    }
    
    /* Umm... compile a probe ;)
     */
    public static boolean compileProbe(File prbJava) {
        String title = "MetaCoretex Probe Compiler";
        String mess;
        int stat=1;
        Object javac = null;
        String[] args = {"-classpath",
        (String)Init.config.get("probe.compiler.classpath"),
        prbJava.getPath()};
        
        try {
            Class compClass = Class.forName("com.sun.tools.javac.Main");
            javac = compClass.newInstance();
        } catch (Exception e) {
            Init.doError("Compiler Error", new Exception("Compiler Classes Not-Found"));
            return(false);
        }
        
        try {
            stat = ((com.sun.tools.javac.Main)javac).compile(args);
        } catch (Exception e) {
            Init.doError("Compiler Error", e);
        }
        
        switch(stat) {
            case 0:
                int reply = Init.askYesNo("Compile Complete", "Compile Successful!\nWould you like to load your new probe?");
                if (reply == JOptionPane.YES_OPTION) {
                    String prbClass = prbJava.getPath().replaceAll("\\.java",".class");
                    Probe prb = loadProbeFromFile(new File(prbClass));
                    
                    System.out.println("Loading :"+prbClass);
                    if (prb != null) {
                        Init.depVect.add(prb);
                        Init.processDepVect();
                    }
                }
                return(true);
                
            case 1:
                mess = "Compiler Error!";
                break;
                
            default:
                mess = "WTF did you do?!?!?";
        }
        Init.doError(mess,new Exception("Unknown Exception"));
        return(false);
    }
}
