/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * UpdateToolz.java
 *
 * Created on June 5, 2003, 2:55 AM
 */

package com.securitycentric.metacoretex.lib;

import com.securitycentric.metacoretex.xml.update.*;
import com.securitycentric.metacoretex.*;
import com.securitycentric.metacoretex.interfaces.*;

import javax.xml.bind.*;
import java.io.*;
import java.util.*;
import java.util.jar.*;
import java.net.*;
import javax.swing.JOptionPane;


/**
 *
 * @author  visigoth
 */
public class UpdateToolz {
    
    public static void onlyProbes(MCTXUpdate upDate) {
        upDate.getConfFile().clear();
        upDate.getJar().clear();
    }
    
    public static void onlyConfigFiles(MCTXUpdate upDate) {
        upDate.getProbe().clear();
        upDate.getJar().clear();
    }
    
    public static void onlyJars(MCTXUpdate upDate) {
        upDate.getProbe().clear();
        upDate.getConfFile().clear();
    }
    
    public static MCTXUpdate getUpdate() throws UpdateException {
        try {
            /*
            SSLSocketFactory sslFact = (SSLSocketFactory)SSLSocketFactory.getDefault();
            SSLSocket s = (SSLSocket)sslFact.createSocket("xml.securitycentric.com", port);
             */
            String updateUrlStr = (String)Init.config.get("update.url");
            JAXBContext jc = JAXBContext.newInstance("com.securitycentric.metacoretex.xml.update");
            Unmarshaller um = jc.createUnmarshaller();
            URL updateUrl = new URL(updateUrlStr);
            MCTXUpdate upDate = (MCTXUpdate)um.unmarshal(updateUrl);
            return(upDate);
            
        } catch (JAXBException e) {
            throw(new UpdateException("JAXB Exception in getUpdate(): \n"+e.getMessage()));
        } catch (MalformedURLException e) {
            throw(new UpdateException("Malforemed URL in getUpdate(): \n"+e.getMessage()));
        }
    }
    
    public static void removeDuplicates(MCTXUpdate upDate) {
        try {
            String confDir = (String)Init.config.get("conf.dir");
            String jarDir = (String)Init.config.get("jar.dir");
            
            // Process probe updates
            Iterator prIter = upDate.getProbe().iterator();
            while (prIter.hasNext()) {
                MCTXProbe prb = (MCTXProbe)prIter.next();
                int id = prb.getId();
                String jName = prb.getJavaName();
                Probe ourPrb = ProbeToolz.getProbeById(id);
                
                // if we have it, test if it's newer
                if (ourPrb != null) {
                    String availVerStr = prb.getVersion();
                    Version availVer = new Version(availVerStr);
                    Version ourVer = new Version(ourPrb.getVersion());
                    
                    if (! availVer.isNewerThan(ourVer))
                        prIter.remove();
                }
            }
            
            // Proces Config Files
            Iterator cnfIter = upDate.getConfFile().iterator();
            while(cnfIter.hasNext()) {
                MCTXConfFile cnfFile = (MCTXConfFile)cnfIter.next();
                String fName = cnfFile.getName();
                File ourCnf = new File(confDir+fName);
                
                if (ourCnf.exists() ) {
                    
                    String availVerStr = cnfFile.getVersion();
                    Version ourVer = getConfVersion(ourCnf);
                    
                    // This probably means they have modified their config,
                    // lets not overwrite it...
                    if (ourVer == null)
                        upDate.getConfFile().remove(cnfFile);
                    
                    Version availVer = new Version(availVerStr);
                    
                    if (! availVer.isNewerThan(ourVer))
                        cnfIter.remove();
                    
                }
                
            }
            
            // Process Jar Files
            Iterator jarIter = upDate.getJar().iterator();
            while (jarIter.hasNext()) {
                MCTXJar MCTXjar= (MCTXJar)jarIter.next();
                String jName = MCTXjar.getName();
                File ourJar = new File(jarDir+jName);
                
                if (ourJar.exists()) {
                    JarFile jarFile = new JarFile(ourJar);
                    Attributes attr = jarFile.getManifest().getMainAttributes();
                    Version ourVer = new Version((String)attr.getValue("MCTX-Jar-Version"));
                    Version availVer =new Version((String)MCTXjar.getVersion());
                    
                    if (! availVer.isNewerThan(ourVer))
                        jarIter.remove();
                }
            }
            
        } catch (VersionException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    /** Carry out the update activities based on the provided MCTXUpdate
     *  which then allows the modification of the MCTXUpdate to suit
     *  contextual needs (duplicates should be removed already).  
     *  Also, reset counter junk at beginning of call
     */
    public static int doUpdate(MCTXUpdate update) throws UpdateException {
        
        newFiles.clear();
        upFiles.clear();
        
        String dlUrlStr = update.getDownloadUrl();
        String confDir = (String)Init.config.get("conf.dir");
        String jarDir = (String)Init.config.get("jar.dir");
        
        // Process probe updates
        Iterator prIter = update.getProbe().iterator();
        while (prIter.hasNext()) {
            MCTXProbe prb = (MCTXProbe)prIter.next();
            
            if (! prb.isUpdate())
                continue;
            
            String jName = prb.getJavaName();
            dlProbe(dlUrlStr,jName);
        }
        
        // Process Jar Files
        Iterator jarIter = update.getJar().iterator();
        while (jarIter.hasNext()) {
            MCTXJar MCTXjar = (MCTXJar)jarIter.next();
            
            if (! MCTXjar.isUpdate())
                continue;
            String jName = MCTXjar.getName();
            dlJarFile(dlUrlStr, jName);
        }
        
        // Proces Config Files
        Iterator cnfIter = update.getConfFile().iterator();
        while(cnfIter.hasNext()) {
            MCTXConfFile cnfFile = (MCTXConfFile)cnfIter.next();
            
            if (! cnfFile.isUpdate())
                continue;
            
            String fName = cnfFile.getName();
            dlConfFile(dlUrlStr, fName);
        }
        
        
        return(newFiles.size()+upFiles.size());
    }
    
    public static void dlProbe(String dlUrlStr, String jName) throws UpdateException {
        try {
            String probeDir = (String) Init.config.get("probe.dir");
            
            URL url = new URL(dlUrlStr+"/probes/"+jName+".class");
            
            dlFile(url,probeDir+"/"+jName+".class");
            
            url = new URL(dlUrlStr+"/probes/"+jName+".java");
            
            dlFile(url,probeDir+"/"+jName+".java");
            
            Probe prb = ProbeToolz.loadProbeFromFile(new File(probeDir+"/"+jName+".class"));
            ProbeToolz.removeProbe(prb.getProbeId());
            int index = ProbeToolz.getIndex(Init.allProbeVect, prb);
            if (index != -1) {
                Init.loadProbe(prb, index);
            }
            
            
        } catch (MalformedURLException e) {
            throw(new UpdateException("Malformed Download URL"));
        }
    }
    
    public static void dlConfFile(String dlUrlStr, String fName) throws UpdateException {
        try {
            String confDir = (String)Init.config.get("conf.dir");
            URL url = new URL(dlUrlStr+"/conf/"+fName);
            dlFile(url, confDir+"/"+fName);
            
        } catch (MalformedURLException e) {
            throw(new UpdateException("Malformed Download URL"));
        }
    }
    
    public static void dlJarFile(String dlUrlStr, String jName) throws UpdateException {
        
        try {
            String jarDir = (String)Init.config.get("jar.dir");
            URL url = new URL(dlUrlStr+"/dist/"+jName);
            dlFile(url, jarDir+"/"+jName);
        } catch (MalformedURLException e) {
            throw(new UpdateException("Malformed Download URL"));
        }
    }
    
    public static void dlFile(URL fLoc, String destFile) {
        // FIXME add copyback?
        try {
            int red;
            byte[] buf = new byte[256];
            InputStream flIn = fLoc.openStream();
            File dest = new File(destFile);
            if (dest.exists()) {
                //File bak = new File(dest.getPath()+".bak");
                upFiles.add(dest.getName());
            } else {
                newFiles.add(dest.getName());
                dest.createNewFile();
            }
            
            FileOutputStream fos = new FileOutputStream(dest);
            
            while ((red=flIn.read(buf)) != -1) {
                fos.write(buf,0,red);
            }
            
        } catch (IOException e) {
            Init.doError("File Download Failed",e);
        }
        
    }
    
    public static Version getConfVersion(File file) {
        try {
            BufferedReader br = new BufferedReader(new FileReader(file));
            // First line should contain version
            String line = br.readLine().trim();
            Version ver = new Version(line);
            return(ver);
            
        } catch (IOException e) {
            e.printStackTrace();
        } catch (VersionException e) {
            e.printStackTrace();
        }
        return(null);
    }
    
    public static Vector newFiles = new Vector();
    public static Vector upFiles = new Vector();
}
