/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * Version.java
 *
 * Created on May 4, 2003, 11:45 AM
 *
 * $Id: Version.java,v 1.4 2003/06/24 05:02:32 visigoth_6 Exp $
 * 
 */

package com.securitycentric.metacoretex.lib;

import java.util.*;
import java.util.regex.*;


/** The Version object can be used by Probes to quickly determine if
 * a version is greater or less than itself.
 *
 * @author visigoth
 */
public class Version {
    
    /** Creates a new instance of Version by parsing the
     *  specified string for a versionish segment of numbers
     *  seperated by [._]
     * @param verStr String representation of the version (ie. 1.354.3_3)
     * @throws VersionException VersionException is thrown if specified string for constructor cannot be
     * parsed to find a version string.
     */
    public Version(String verStr) throws VersionException {

        pat = Pattern.compile("[\\d\\._]{3,}");
        m = pat.matcher(verStr);

        if ( m.find()) {
            verString = m.group();
            //System.out.println("new Version split :" + verString);
            String[] strAry = verString.split("[\\._]+");
            verNums = new int[strAry.length];

            for (int i = 0; i < strAry.length; i++) {
                try {
                    verNums[i] = Integer.parseInt(strAry[i]);
                } catch (NumberFormatException ne) {
                    throw(new VersionException("Column " + i + " seems malformed: "+strAry[i]));
                }
            }

        } else {
            throw(new VersionException("Couldn't find version numbers in string"));
        }
    }

    /** Tests to see if this version is newer than the specified
     *  version object (non-Inclusive).
     * @param ver Version object to compare to
     * @return boolean true if ver is newer than this probe. False if not
     */
    public boolean isNewerThan(Version ver) {
        for (int i = 0; i < ver.verCount(); i++) {
            int us = verInt(i);
            int them = ver.verInt(i);

            if (us > them)
                return(true);

            if (them > us)
                return(false);
        }
        return(false);
    }

    /** Tests to see if this version is older than the specified
     *  version object (non-Inclusive).
     */
    public boolean isOlderThan(Version ver) {
        for (int i = 0; i < ver.verCount(); i++) {
            int us = verInt(i);
            int them = ver.verInt(i);
            
            //System.out.println("Compairing "+us+" to "+them+"\n");
            if (them > us)
                return(true);
            
            if (us > them)
                return(false);
        }
        return(false);
    }

    /** Returns the integer value of the specified index in the underlying version
     * string.  ie. the verInt(3) for the version 3.145.42.4_3 would be 42.
     */    
    public int verInt(int index) {
        if (index >= verNums.length)
            return(0);

        return(verNums[index]);
    }

    /** Returns the number of indices in the version string.
     * ie. 4.24.64_2 would return 4
     */    
    public int verCount() {
        return(verNums.length);
    }

    /** Returns the string representation of this version in dot notation.
     */    
    public String toString() {
        return(verString);
    }

    private String verString;
    private int[] verNums;
    private Pattern pat;
    private Matcher m;
}
